@@ -3,7 +3,7 @@ TdGsA <- function(fcasts, bts, topts) {
   # Top-down forecasts based on the average historical proportions. (Gross-Sohl
   # method A)
   div <- apply(bts, 2, function(x) x/topts)
-  prop <- apply(div, 2, sum, na.rm = TRUE)/length(time(bts))
+  prop <- colMeans(div, na.rm = TRUE)
   out <- fcasts %*% prop
   return(out)
 }
@@ -11,7 +11,7 @@ TdGsA <- function(fcasts, bts, topts) {
 TdGsF <- function(fcasts, bts, topts) {
   # Top-down forecasts based on the proportions of the historical averages (
   # Gross-Sohl method F)
-  numerator <- apply(bts, 2, sum, na.rm = TRUE)
+  numerator <- colSums(bts, na.rm = TRUE)
   denominator <- sum(topts, na.rm = TRUE)
   prop <- numerator/denominator
   out <- fcasts %*% prop
